@@ -361,6 +361,8 @@ define_parser!(ParseValue, Value, |_, state| parse_first_of!(state, {
     ParseFunctionCall => Value::FunctionCall,
     ParseVar => Value::Var,
     ParseParenExpression => Value::ParenthesesExpression,
+    @#[cfg(feature = "roblox")]
+    ParseIfExpression => Value::IfExpression,
 }));
 
 #[derive(Clone, Debug, Default, PartialEq)]
@@ -741,6 +743,14 @@ define_parser!(ParseCall, Call, |_, state| parse_first_of!(state, {
 struct ParseFunctionBody;
 #[rustfmt::skip]
 define_parser!(ParseFunctionBody, FunctionBody, |_, state| {
+    #[cfg(feature = "roblox")]
+    let (state, generics) =
+        if let Ok((state, generics)) = keep_going!(ParseGenericDeclaration.parse(state)) {
+            (state, Some(generics))
+        } else {
+            (state, None)
+        };
+
     let (mut state, start_parenthese) = expect!(
         state,
         ParseSymbol(Symbol::LeftParen).parse(state),
@@ -790,7 +800,7 @@ define_parser!(ParseFunctionBody, FunctionBody, |_, state| {
 
                 // Parse ellipse type if in Luau
                 if cfg!(feature = "roblox") {
-                    if let Ok((new_state, type_specifier)) = ParseTypeSpecifier(TypeInfoContext::None).parse(state) {
+                    if let Ok((new_state, type_specifier)) = ParseTypeSpecifier(TypeInfoContext::VarArgSpecifier).parse(state) {
                         state = new_state;
                         type_specifiers.push(Some(type_specifier));
                     } else {
@@ -805,7 +815,7 @@ define_parser!(ParseFunctionBody, FunctionBody, |_, state| {
 
         // Parse ellipse type if in Luau
         if cfg!(feature = "roblox") {
-            if let Ok((new_state, type_specifier)) = ParseTypeSpecifier(TypeInfoContext::None).parse(state) {
+            if let Ok((new_state, type_specifier)) = ParseTypeSpecifier(TypeInfoContext::VarArgSpecifier).parse(state) {
                 state = new_state;
                 type_specifiers.push(Some(type_specifier));
             } else {
@@ -836,6 +846,8 @@ define_parser!(ParseFunctionBody, FunctionBody, |_, state| {
     Ok((
         state,
         FunctionBody {
+            #[cfg(feature = "roblox")]
+            generics,
             parameters_parentheses: ContainedSpan::new(start_parenthese, end_parenthese),
             parameters,
             block,
@@ -914,13 +926,6 @@ define_parser!(ParseLocalFunction, LocalFunction, |_, state| {
     let (state, local_token) = ParseSymbol(Symbol::Local).parse(state)?;
     let (state, function_token) = ParseSymbol(Symbol::Function).parse(state)?;
     let (state, name) = expect!(state, ParseIdentifier.parse(state), "expected name");
-    #[cfg(feature = "roblox")]
-    let (state, generics) =
-        if let Ok((state, generics)) = keep_going!(ParseGenericDeclaration.parse(state)) {
-            (state, Some(generics))
-        } else {
-            (state, None)
-        };
     let (state, body) = ParseFunctionBody.parse(state)?;
     Ok((
         state,
@@ -928,8 +933,6 @@ define_parser!(ParseLocalFunction, LocalFunction, |_, state| {
             local_token,
             function_token,
             name,
-            #[cfg(feature = "roblox")]
-            generics,
             body,
         },
     ))
@@ -1055,13 +1058,6 @@ define_parser!(ParseFunctionDeclaration, FunctionDeclaration, |_, state| {
         ParseFunctionName.parse(state),
         "expected function name"
     );
-    #[cfg(feature = "roblox")]
-    let (state, generics) =
-        if let Ok((state, generics)) = keep_going!(ParseGenericDeclaration.parse(state)) {
-            (state, Some(generics))
-        } else {
-            (state, None)
-        };
     let (state, body) = expect!(
         state,
         ParseFunctionBody.parse(state),
@@ -1072,8 +1068,6 @@ define_parser!(ParseFunctionDeclaration, FunctionDeclaration, |_, state| {
         FunctionDeclaration {
             function_token,
             name,
-            #[cfg(feature = "roblox")]
-            generics,
             body,
         },
     ))
@@ -1096,14 +1090,22 @@ define_parser!(ParseIdentifier, TokenReference, |_, state| {
 #[derive(Clone, Copy, Debug, PartialEq)]
 enum TypeInfoContext {
     /// A standard type info, with no context
+    #[cfg(feature = "roblox")]
     None,
     /// A type inside of parentheses, either for the parameters in a `TypeInfo::Callback`, or for a `TypeInfo::Tuple`
     /// Variadic type infos are only permitted inside of here
     #[cfg(feature = "roblox")]
     ParenthesesType,
     /// The return type of a function declaration or callback type, such as `function foo(bar) -> number`.
-    /// This is the only location where Tuple types are allowed. Variadic type infos are also allowed here
+    /// In addition to standard types, type packs/variadic type packs are allowed here
     ReturnType,
+    /// A type specifier for the variadic argument `...` in a function definition parameter list
+    /// In these cases, we are allowed a generic variadic pack `T...` to be specified
+    VarArgSpecifier,
+    /// An argument passed into a generic type, e.g. `string` or `...number` in `type X = Foo<string, ...number>`.
+    /// In these cases, generic type packs, variadic type packs, and explicit type packs (using parentheses) are allowed
+    #[cfg(feature = "roblox")]
+    GenericArgument,
 }
 
 // Roblox Types
@@ -1159,14 +1161,86 @@ define_roblox_parser!(
     TokenReference,
     |_, state| {
         let (state, start_arrow) = ParseSymbol(Symbol::LessThan).parse(state)?;
-        let (state, generics) = expect!(
-            state,
-            OneOrMore(ParseIdentifier, ParseSymbol(Symbol::Comma), false).parse(state),
-            "expected type parameters"
-        );
+        let mut generics: Punctuated<GenericDeclarationParameter> = Punctuated::new();
+
+        let mut have_seen_pack = false; // We have seen a generic type pack parameter. We can only now accept generic type pack parameters
+        let mut have_seen_default = false; // We have seen a default type, all future parameters must define a default type
+
+        let mut loop_state = state;
+        loop {
+            let (state, name) = expect!(
+                loop_state,
+                ParseIdentifier.parse(loop_state),
+                "expected name"
+            );
+
+            // Look to see if is a type pack
+            let (state, ellipse) = if let Ok((state, ellipse)) =
+                keep_going!(ParseSymbol(Symbol::Ellipse).parse(state))
+            {
+                have_seen_pack = true;
+                (state, Some(ellipse))
+            } else {
+                // Must only be type packs after we have seen one
+                if have_seen_pack {
+                    return Err(InternalAstError::UnexpectedToken {
+                        token: state.peek().clone(),
+                        additional: Some("generic types come before generic type packs".into()),
+                    });
+                };
+
+                (state, None)
+            };
+
+            // Look to see if a default type has been sepcified
+            let (state, default) =
+                if let Ok((state, equals)) = keep_going!(ParseSymbol(Symbol::Equal).parse(state)) {
+                    have_seen_default = true;
+
+                    // If we are parsing a type pack, allow type pack default types
+                    let parse_context = if ellipse.is_some() {
+                        TypeInfoContext::GenericArgument
+                    } else {
+                        TypeInfoContext::None
+                    };
+
+                    let (state, type_info) = expect!(
+                        state,
+                        ParseTypeInfo(parse_context).parse(state),
+                        "expected type after `=`"
+                    );
+
+                    (state, Some((equals, type_info)))
+                } else {
+                    // Defaults must always be specified once one has been
+                    if have_seen_default {
+                        return Err(InternalAstError::UnexpectedToken {
+                            token: state.peek().clone(),
+                            additional: Some("expected default type after type name".into()),
+                        });
+                    }
+                    (state, None)
+                };
+
+            let parameter = match ellipse {
+                Some(ellipse) => GenericParameterInfo::Variadic { name, ellipse },
+                None => GenericParameterInfo::Name(name),
+            };
+            let declaration_parameter = GenericDeclarationParameter { parameter, default };
+
+            if let Ok((state, punctuation)) = keep_going!(ParseSymbol(Symbol::Comma).parse(state)) {
+                generics.push(Pair::Punctuated(declaration_parameter, punctuation));
+                loop_state = state;
+            } else {
+                generics.push(Pair::End(declaration_parameter));
+                loop_state = state;
+                break;
+            }
+        }
+
         let (state, end_arrow) = expect!(
-            state,
-            ParseSymbol(Symbol::GreaterThan).parse(state),
+            loop_state,
+            ParseSymbol(Symbol::GreaterThan).parse(loop_state),
             "expected `>` to match `<`"
         );
 
@@ -1208,6 +1282,58 @@ cfg_if::cfg_if! {
             }
         );
 
+        // Roblox If Expression
+        #[derive(Clone, Debug, Default, PartialEq)]
+        struct ParseIfExpression;
+        define_parser!(
+            ParseIfExpression,
+            IfExpression,
+            |_, state| {
+                let (state, if_token) = ParseSymbol(Symbol::If).parse(state)?;
+                let (state, condition) = expect!(state, ParseExpression.parse(state), "expected condition");
+                let (state, then_token) = expect!(state, ParseSymbol(Symbol::Then).parse(state), "expected `then`");
+                let (mut state, if_expression) = expect!(state, ParseExpression.parse(state), "expected expression");
+
+                let mut else_if_expressions = Vec::new();
+                while let Ok((new_state, else_if_token)) = ParseSymbol(Symbol::ElseIf).parse(state) {
+                    let (new_state, condition) = expect!(
+                        state,
+                        ParseExpression.parse(new_state),
+                        "expected condition"
+                    );
+                    let (new_state, then_token) = expect!(
+                        state,
+                        ParseSymbol(Symbol::Then).parse(new_state),
+                        "expected 'then'"
+                    );
+                    let (new_state, expression) = expect!(state, ParseExpression.parse(new_state), "expected expression");
+                    state = new_state;
+                    else_if_expressions.push(ElseIfExpression {
+                        else_if_token,
+                        condition,
+                        then_token,
+                        expression,
+                    });
+                }
+
+                let (state, else_token) = expect!(state, ParseSymbol(Symbol::Else).parse(state), "expected `else` in if expression");
+                let (state, else_expression) = expect!(state, ParseExpression.parse(state), "expected expression");
+
+                Ok((
+                    state,
+                    IfExpression {
+                        if_token,
+                        condition,
+                        then_token,
+                        if_expression,
+                        else_if_expressions: if else_if_expressions.is_empty() { None } else { Some(else_if_expressions) },
+                        else_token,
+                        else_expression,
+                    },
+                ))
+            }
+        );
+
         #[derive(Clone, Debug, PartialEq)]
         struct ParseTypeDeclaration;
         define_parser!(
@@ -1332,6 +1458,26 @@ cfg_if::cfg_if! {
             }))
         });
 
+        #[derive(Clone, Debug, PartialEq)]
+        /// A tuple type info, such as `(string, foo)` in `(string) -> (string, foo)`.
+        /// This is only used for return types. An opening parentheses should have already been consumed,
+        /// and is passed to this struct.
+        struct ParseTupleTypeInfo(TokenReference);
+        define_parser!(ParseTupleTypeInfo, TypeInfo, |this, state| {
+            let (state, types) = expect!(
+                state,
+                ZeroOrMoreDelimited(ParseTypeInfo(TypeInfoContext::ParenthesesType), ParseSymbol(Symbol::Comma), false)
+                    .parse(state),
+                "expected types within parentheses"
+            );
+            let (state, end_parenthese) = expect!(state, ParseSymbol(Symbol::RightParen).parse(state), "expected `)`");
+
+            Ok((state, TypeInfo::Tuple {
+                parentheses: ContainedSpan::new(this.0.clone(), end_parenthese),
+                types,
+            }))
+        });
+
         /// A type array atom, such as `{ string }`.
         /// An opening brace should have already been consumed, and is passed to this struct.
         #[derive(Clone, Debug, PartialEq)]
@@ -1389,7 +1535,7 @@ cfg_if::cfg_if! {
         #[derive(Clone, Debug, PartialEq)]
         /// A callback type info atom, such as `(count: number) -> string` in `type Foo = (count: number) -> string`.
         /// An opening parentheses should have already been consumed, and is passed to this struct.
-        struct ParseCallbackTypeInfo(TokenReference);
+        struct ParseCallbackTypeInfo(TokenReference, Option<GenericDeclaration>);
         define_parser!(ParseCallbackTypeInfo, TypeInfo, |this, state| {
             let (state, types) = expect!(
                 state,
@@ -1414,6 +1560,7 @@ cfg_if::cfg_if! {
             Ok((
                 state,
                 TypeInfo::Callback {
+                    generics: this.1.clone(),
                     arguments: types,
                     parentheses: ContainedSpan::new(this.0.clone(), end_parenthese),
                     arrow,
@@ -1467,11 +1614,20 @@ cfg_if::cfg_if! {
         #[derive(Clone, Debug, PartialEq)]
         struct ParseSingleTypeInfo(TypeInfoContext);
         define_parser!(ParseSingleTypeInfo, TypeInfo, |this, state| {
-            let (state, base_type) = if let Ok((state, identifier)) = {
-                ParseIdentifier
+            // Singleton type info: `"yes" | "no"` in `(string) -> "yes" | "no"`, `"$$typeof"` in `{ ["$$typeof"]: number }`,
+            let (state, base_type) = if let Ok((state, string_singleton)) = ParseStringLiteral.parse(state) {
+                (state, TypeInfo::String(string_singleton))
+            // Singleton type info: `true` in `type X = Error & { handled: true }`
+            } else if let Ok((state, true_singleton)) = ParseSymbol(Symbol::True).parse(state) {
+                (state, TypeInfo::Boolean(true_singleton))
+            } else if let Ok((state, false_singleton)) = ParseSymbol(Symbol::False).parse(state) {
+                (state, TypeInfo::Boolean(false_singleton))
+            // Singleton type info: `nil` in `local function get(x: string, y: nil) end`
+            } else if let Ok((state, nil_singleton)) = ParseSymbol(Symbol::Nil).parse(state) {
+                (state, TypeInfo::Basic(nil_singleton))
+            } else if let Ok((state, identifier)) = ParseIdentifier
                     .parse(state)
-                    .or_else(|_| ParseSymbol(Symbol::Nil).parse(state))
-            } {
+            {
                 if identifier.token().to_string() == "typeof" {
                     let (state, start_parenthese) = expect!(
                         state,
@@ -1519,7 +1675,7 @@ cfg_if::cfg_if! {
                 {
                     let (state, generics) = expect!(
                         state,
-                        OneOrMore(ParseTypeInfo(TypeInfoContext::None), ParseSymbol(Symbol::Comma), false).parse(state),
+                        OneOrMore(ParseTypeInfo(TypeInfoContext::GenericArgument), ParseSymbol(Symbol::Comma), false).parse(state),
                         "expected type parameters"
                     );
 
@@ -1537,13 +1693,36 @@ cfg_if::cfg_if! {
                             generics,
                         },
                     )
+                } else if matches!(this.0, TypeInfoContext::ParenthesesType | TypeInfoContext::ReturnType | TypeInfoContext::VarArgSpecifier | TypeInfoContext::GenericArgument) {
+                    // Check for a generic type pack
+                    if let Ok((state, ellipse)) = ParseSymbol(Symbol::Ellipse).parse(state) {
+                        (state, TypeInfo::GenericPack {
+                            name: identifier,
+                            ellipse
+                        })
+                    } else {
+                        (state, TypeInfo::Basic(identifier))
+                    }
                 } else {
                     (state, TypeInfo::Basic(identifier))
                 }
+            } else if let Ok((state, generics)) = ParseGenericDeclaration.parse(state) {
+                // Callback with a generic type
+                let (state, start_parenthese) = ParseSymbol(Symbol::LeftParen).parse(state)?;
+                ParseCallbackTypeInfo(start_parenthese, Some(generics)).parse(state)?
             } else if let Ok((state, start_parenthese)) =
                 ParseSymbol(Symbol::LeftParen).parse(state)
             {
-                if let Ok((state, parentheses_type)) = ParseParenthesesTypeInfo(start_parenthese.clone()).parse(state) {
+                // Parse types encapsulated in parentheses as an atom. If we are allowing type packs (i.e. as a return type or a generic type argument),
+                // this could be a tuple, otherwise this can only be a singular type specified within parentheses.
+                let atom =
+                    if matches!(this.0, TypeInfoContext::ReturnType | TypeInfoContext::GenericArgument) {
+                        ParseTupleTypeInfo(start_parenthese.clone()).parse(state)
+                    } else {
+                        ParseParenthesesTypeInfo(start_parenthese.clone()).parse(state)
+                    };
+
+                if let Ok((state, parentheses_type)) = atom {
                     // Single token lookahead: see if we have a `->` ahead. If we do, we need to parse this
                     // as a callback type, using what we already have from the parentheses type
                     if let Ok((state, arrow)) = ParseSymbol(Symbol::ThinArrow).parse(state) {
@@ -1570,6 +1749,7 @@ cfg_if::cfg_if! {
                         (
                             state,
                             TypeInfo::Callback {
+                                generics: None,
                                 arguments,
                                 parentheses,
                                 arrow,
@@ -1580,7 +1760,7 @@ cfg_if::cfg_if! {
                         (state, parentheses_type)
                     }
                 } else {
-                    ParseCallbackTypeInfo(start_parenthese).parse(state)?
+                    ParseCallbackTypeInfo(start_parenthese, None).parse(state)?
                 }
             } else if let Ok((state, start_brace)) = ParseSymbol(Symbol::LeftBrace).parse(state) {
                 if let Ok((state, type_array)) = ParseTypeArray(start_brace.clone()).parse(state) {
@@ -1588,6 +1768,20 @@ cfg_if::cfg_if! {
                 } else {
                     ParseTypeTable(start_brace).parse(state)?
                 }
+            } else if matches!(this.0, TypeInfoContext::GenericArgument) {
+                // Only allowed variadic type packs as a generic argument
+                // Note, this is `...T`, but `T` is a token, not a type info (e.g. `...string` is allowed but `...{}` is not)
+                if let Ok((state, ellipse)) = ParseSymbol(Symbol::Ellipse).parse(state) {
+                    let (state, name) = expect!(
+                        state,
+                        ParseIdentifier.parse(state),
+                        "expected name after `...`"
+                    );
+
+                    (state, TypeInfo::VariadicPack { ellipse, name })
+                } else {
+                    return Err(InternalAstError::NoMatch);
+                }
             } else if matches!(this.0, TypeInfoContext::ParenthesesType | TypeInfoContext::ReturnType) {
                 // Only allow variadic type annotation for a return type or a tuple type
                 if let Ok((state, ellipse)) = ParseSymbol(Symbol::Ellipse).parse(state) {
@@ -1614,74 +1808,10 @@ cfg_if::cfg_if! {
             Ok((state, base_type))
         });
 
-        #[derive(Clone, Debug, PartialEq)]
-        /// Similar to [`ParseSingleTypeInfo`], except it allows parsing of tuple types `(x, y)`.
-        /// Should only be used when parsing a return type from a function
-        struct ParseSingleReturnTypeInfo;
-        define_parser!(ParseSingleReturnTypeInfo, TypeInfo, |_, state| {
-            // Attempt to parse a tuple type.
-            if let Ok((state, start_parenthese)) =
-                ParseSymbol(Symbol::LeftParen).parse(state)
-            {
-                let (state, types) = expect!(
-                    state,
-                    ZeroOrMoreDelimited(ParseTypeInfo(TypeInfoContext::ParenthesesType), ParseSymbol(Symbol::Comma), false)
-                        .parse(state),
-                    "expected types within parentheses"
-                );
-
-                let (state, end_parenthese) = expect!(
-                    state,
-                    ParseSymbol(Symbol::RightParen).parse(state),
-                    "expected `)` to match `(`"
-                );
-
-                // Single token lookahead: if we see a `->` afterwards, we need to bail out
-                // of attempting to parse a tuple type, since it must be a callback.
-                if let Ok((state, arrow)) = ParseSymbol(Symbol::ThinArrow).parse(state) {
-                    // `types` is currently `Punctuated<TypeInfo>`, but we need to map it to `Punctuated<TypeArgument>`
-                    // where each argument has no name.
-                    let arguments = types.into_pairs().map(|pair| pair.map(|type_info| TypeArgument {
-                        name: None,
-                        type_info
-                    })).collect::<Punctuated<_>>();
-
-                    let (state, return_value) = expect!(
-                        state,
-                        ParseTypeInfo(TypeInfoContext::ReturnType).parse(state),
-                        "expected return type after `->`"
-                    );
-
-                    Ok((
-                        state,
-                        TypeInfo::Callback {
-                            arguments,
-                            parentheses: ContainedSpan::new(start_parenthese, end_parenthese),
-                            arrow,
-                            return_type: Box::new(return_value),
-                        },
-                    ))
-                } else {
-                    Ok((
-                        state,
-                        TypeInfo::Tuple {
-                            parentheses: ContainedSpan::new(start_parenthese, end_parenthese),
-                            types,
-                        },
-                    ))
-                }
-            } else {
-                ParseSingleTypeInfo(TypeInfoContext::ReturnType).parse(state)
-            }
-        });
-
         #[derive(Clone, Debug, PartialEq)]
         struct ParseSingleTypeInfoAtom(TypeInfoContext);
         define_parser!(ParseSingleTypeInfoAtom, TypeInfo, |this, state| {
-            let (mut state, mut base_type) = match this.0 {
-                TypeInfoContext::ReturnType => ParseSingleReturnTypeInfo.parse(state)?,
-                _ => ParseSingleTypeInfo(this.0).parse(state)?,
-            };
+            let (mut state, mut base_type) = ParseSingleTypeInfo(this.0).parse(state)?;
 
             if let Ok((new_state, question_mark)) = ParseSymbol(Symbol::QuestionMark).parse(state) {
                 base_type = TypeInfo::Optional {
