@@ -20,12 +20,20 @@ impl Visit for TypeInfo {
             TypeInfo::Basic(__self_0) => {
                 __self_0.visit(visitor);
             }
+            TypeInfo::String(__self_0) => {
+                __self_0.visit(visitor);
+            }
+            TypeInfo::Boolean(__self_0) => {
+                __self_0.visit(visitor);
+            }
             TypeInfo::Callback {
+                generics,
                 parentheses,
                 arguments,
                 arrow,
                 return_type,
             } => {
+                generics.visit(visitor);
                 parentheses.tokens.0.visit(visitor);
                 arguments.visit(visitor);
                 parentheses.tokens.1.visit(visitor);
@@ -42,6 +50,10 @@ impl Visit for TypeInfo {
                 generics.visit(visitor);
                 arrows.tokens.1.visit(visitor);
             }
+            TypeInfo::GenericPack { name, ellipse } => {
+                name.visit(visitor);
+                ellipse.visit(visitor);
+            }
             TypeInfo::Module {
                 module,
                 punctuation,
@@ -96,6 +108,10 @@ impl Visit for TypeInfo {
                 ellipse.visit(visitor);
                 type_info.visit(visitor);
             }
+            TypeInfo::VariadicPack { ellipse, name } => {
+                ellipse.visit(visitor);
+                name.visit(visitor);
+            }
         };
         visitor.visit_type_info_end(self);
     }
@@ -116,12 +132,16 @@ impl VisitMut for TypeInfo {
                 TypeInfo::Array { braces, type_info }
             }
             TypeInfo::Basic(__self_0) => TypeInfo::Basic(__self_0.visit_mut(visitor)),
+            TypeInfo::Boolean(__self_0) => TypeInfo::Boolean(__self_0.visit_mut(visitor)),
+            TypeInfo::String(__self_0) => TypeInfo::String(__self_0.visit_mut(visitor)),
             TypeInfo::Callback {
+                mut generics,
                 mut parentheses,
                 mut arguments,
                 mut arrow,
                 mut return_type,
             } => {
+                generics = generics.visit_mut(visitor);
                 parentheses.tokens.0 = parentheses.tokens.0.visit_mut(visitor);
                 arguments = arguments.visit_mut(visitor);
                 parentheses.tokens.1 = parentheses.tokens.1.visit_mut(visitor);
@@ -129,6 +149,7 @@ impl VisitMut for TypeInfo {
                 return_type = return_type.visit_mut(visitor);
 
                 TypeInfo::Callback {
+                    generics,
                     parentheses,
                     arguments,
                     arrow,
@@ -153,6 +174,16 @@ impl VisitMut for TypeInfo {
                 }
             }
 
+            TypeInfo::GenericPack {
+                mut name,
+                mut ellipse,
+            } => {
+                name = name.visit_mut(visitor);
+                ellipse = ellipse.visit_mut(visitor);
+
+                TypeInfo::GenericPack { name, ellipse }
+            }
+
             TypeInfo::Module {
                 mut module,
                 mut punctuation,
@@ -236,6 +267,11 @@ impl VisitMut for TypeInfo {
                 ellipse: ellipse.visit_mut(visitor),
                 type_info: type_info.visit_mut(visitor),
             },
+
+            TypeInfo::VariadicPack { ellipse, name } => TypeInfo::VariadicPack {
+                ellipse: ellipse.visit_mut(visitor),
+                name: name.visit_mut(visitor),
+            },
         };
         self = visitor.visit_type_info_end(self);
         self
