@@ -22,9 +22,18 @@ pub enum TypeInfo {
     #[display(fmt = "{}", "_0")]
     Basic(TokenReference),
 
+    /// A singleton string type, such as `"hello"`
+    #[display(fmt = "{}", "_0")]
+    String(TokenReference),
+
+    /// A singleton boolean type, such as `true`
+    #[display(fmt = "{}", "_0")]
+    Boolean(TokenReference),
+
     /// A callback type, such as `(string, number) => boolean`.
     #[display(
-        fmt = "{}{}{}{}{}",
+        fmt = "{}{}{}{}{}{}",
+        "display_option(generics)",
         "parentheses.tokens().0",
         "arguments",
         "parentheses.tokens().1",
@@ -32,6 +41,8 @@ pub enum TypeInfo {
         "return_type"
     )]
     Callback {
+        /// Optional generics provided for the arguments, such as in `<T>(T) -> string`
+        generics: Option<GenericDeclaration>,
         /// The parentheses for the arguments.
         parentheses: ContainedSpan,
         /// The argument types: `(string, number)`.
@@ -59,6 +70,16 @@ pub enum TypeInfo {
         generics: Punctuated<TypeInfo>,
     },
 
+    /// A generic pack: `T...`.
+    /// Note, these are only available as return types, when annotating a vararg (`...`) in a function parameter, or as a generic type argument.
+    #[display(fmt = "{}{}", "name", "ellipse")]
+    GenericPack {
+        /// The name of the type that is generic: `T`.
+        name: TokenReference,
+        /// The ellipse: `...`.
+        ellipse: TokenReference,
+    },
+
     /// An intersection type: `string & number`, denoting both types.
     #[display(fmt = "{}{}{}", "left", "ampersand", "right")]
     Intersection {
@@ -149,6 +170,15 @@ pub enum TypeInfo {
         /// The type that is variadic: `number`.
         type_info: Box<TypeInfo>,
     },
+
+    /// A variadic type pack: `...T` in `Function<...T>`
+    #[display(fmt = "{}{}", "ellipse", "name")]
+    VariadicPack {
+        /// The ellipse: `...`
+        ellipse: TokenReference,
+        /// The name of the type that is variadic: `T`
+        name: TokenReference,
+    },
 }
 
 /// A subset of TypeInfo that consists of items which can only be used as an index, such as `Foo` and `Foo<Bar>`,
@@ -386,6 +416,73 @@ impl TypeDeclaration {
     }
 }
 
+/// A generic declaration parameter used in [`GenericDeclaration`]. Can either be a name or a variadic pack.
+#[derive(Clone, Debug, Display, PartialEq, Node, Visit)]
+#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
+#[non_exhaustive]
+pub enum GenericParameterInfo {
+    /// A name, such as `foo`.
+    #[display(fmt = "{}", "_0")]
+    Name(TokenReference),
+
+    /// A variadic type pack: `T...`.
+    #[display(fmt = "{}{}", "name", "ellipse")]
+    Variadic {
+        /// The name of the type that is variadic: `T`.
+        name: TokenReference,
+        /// The ellipse: `...`.
+        ellipse: TokenReference,
+    },
+}
+/// A generic declaration parameter us in [`GenericDeclaration`]. Consists of a [`GenericParameterInfo`] and an optional default type.
+#[derive(Clone, Debug, Display, PartialEq, Node, Visit)]
+#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
+#[display(
+    fmt = "{}{}{}",
+    "parameter",
+    "display_option(self.equals())",
+    "display_option(self.default_type())"
+)]
+pub struct GenericDeclarationParameter {
+    pub(crate) parameter: GenericParameterInfo,
+    pub(crate) default: Option<(TokenReference, TypeInfo)>,
+}
+
+impl GenericDeclarationParameter {
+    /// Creates a new GenericDeclarationParameter
+    pub fn new(parameter: GenericParameterInfo) -> Self {
+        Self {
+            parameter,
+            default: None,
+        }
+    }
+
+    /// The generic parameter
+    pub fn parameter(&self) -> &GenericParameterInfo {
+        &self.parameter
+    }
+
+    /// The equals symbol denoting a default type, if present
+    pub fn equals(&self) -> Option<&TokenReference> {
+        self.default.as_ref().map(|(equals, _)| equals)
+    }
+
+    /// The default type, if present
+    pub fn default_type(&self) -> Option<&TypeInfo> {
+        self.default.as_ref().map(|(_, default_type)| default_type)
+    }
+
+    /// Returns a new GenericDeclarationParameter with the given parameter info
+    pub fn with_parameter(self, parameter: GenericParameterInfo) -> Self {
+        Self { parameter, ..self }
+    }
+
+    /// Returns a new GenericDeclarationParameter with the given default type
+    pub fn with_default(self, default: Option<(TokenReference, TypeInfo)>) -> Self {
+        Self { default, ..self }
+    }
+}
+
 /// The generics used in a [`TypeDeclaration`].
 #[derive(Clone, Debug, Display, PartialEq, Node, Visit)]
 #[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
@@ -393,7 +490,7 @@ impl TypeDeclaration {
 pub struct GenericDeclaration {
     #[visit(contains = "generics")]
     pub(crate) arrows: ContainedSpan,
-    pub(crate) generics: Punctuated<TokenReference>,
+    pub(crate) generics: Punctuated<GenericDeclarationParameter>,
 }
 
 impl GenericDeclaration {
@@ -414,7 +511,7 @@ impl GenericDeclaration {
     }
 
     /// The names of the generics: `T, U` in `<T, U>`.
-    pub fn generics(&self) -> &Punctuated<TokenReference> {
+    pub fn generics(&self) -> &Punctuated<GenericDeclarationParameter> {
         &self.generics
     }
 
@@ -424,7 +521,7 @@ impl GenericDeclaration {
     }
 
     /// Returns a new TypeDeclaration with the given names of the generics
-    pub fn with_generics(self, generics: Punctuated<TokenReference>) -> Self {
+    pub fn with_generics(self, generics: Punctuated<GenericDeclarationParameter>) -> Self {
         Self { generics, ..self }
     }
 }
@@ -590,6 +687,21 @@ make_op!(CompoundOp,
     }
 );
 
+impl CompoundOp {
+    /// The token associated with the operator
+    pub fn token(&self) -> &TokenReference {
+        match self {
+            Self::PlusEqual(token)
+            | Self::MinusEqual(token)
+            | Self::StarEqual(token)
+            | Self::SlashEqual(token)
+            | Self::PercentEqual(token)
+            | Self::CaretEqual(token)
+            | Self::TwoDotsEqual(token) => token,
+        }
+    }
+}
+
 /// A Compound Assignment statement, such as `x += 1` or `x -= 1`
 #[derive(Clone, Debug, Display, PartialEq, Node, Visit)]
 #[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
@@ -643,3 +755,197 @@ impl CompoundAssignment {
         Self { rhs, ..self }
     }
 }
+
+/// An if statement
+#[derive(Clone, Debug, Display, PartialEq, Node, Visit)]
+#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
+#[display(
+    fmt = "{}{}{}{}{}{}{}",
+    "if_token",
+    "condition",
+    "then_token",
+    "if_expression",
+    "display_option(else_if_expressions.as_ref().map(join_vec))",
+    "else_token",
+    "else_expression"
+)]
+pub struct IfExpression {
+    pub(crate) if_token: TokenReference,
+    pub(crate) condition: Expression,
+    pub(crate) then_token: TokenReference,
+    pub(crate) if_expression: Expression,
+    pub(crate) else_if_expressions: Option<Vec<ElseIfExpression>>,
+    pub(crate) else_token: TokenReference,
+    pub(crate) else_expression: Expression,
+}
+
+impl IfExpression {
+    /// Creates a new If from the given condition
+    pub fn new(
+        condition: Expression,
+        if_expression: Expression,
+        else_expression: Expression,
+    ) -> Self {
+        Self {
+            if_token: TokenReference::symbol("if ").unwrap(),
+            condition,
+            then_token: TokenReference::symbol(" then").unwrap(),
+            if_expression,
+            else_if_expressions: None,
+            else_token: TokenReference::symbol(" else ").unwrap(),
+            else_expression,
+        }
+    }
+
+    /// The `if` token
+    pub fn if_token(&self) -> &TokenReference {
+        &self.if_token
+    }
+
+    /// The condition of the if expression, `condition` in `if condition then`
+    pub fn condition(&self) -> &Expression {
+        &self.condition
+    }
+
+    /// The `then` token
+    pub fn then_token(&self) -> &TokenReference {
+        &self.then_token
+    }
+
+    /// The expression evaluated if the initial if condition holds
+    pub fn if_expression(&self) -> &Expression {
+        &self.if_expression
+    }
+
+    /// The `else` token
+    pub fn else_token(&self) -> &TokenReference {
+        &self.else_token
+    }
+
+    /// If there are `elseif` conditions, returns a vector of them
+    // TODO: Make this return an iterator, and remove Option part entirely?
+    pub fn else_if_expressions(&self) -> Option<&Vec<ElseIfExpression>> {
+        self.else_if_expressions.as_ref()
+    }
+
+    /// The else expression if all other conditions do not hold
+    pub fn else_expression(&self) -> &Expression {
+        &self.else_expression
+    }
+
+    /// Returns a new IfExpression with the given `if` token
+    pub fn with_if_token(self, if_token: TokenReference) -> Self {
+        Self { if_token, ..self }
+    }
+
+    /// Returns a new IfExpression with the given condition
+    pub fn with_condition(self, condition: Expression) -> Self {
+        Self { condition, ..self }
+    }
+
+    /// Returns a new IfExpression with the given `then` token
+    pub fn with_then_token(self, then_token: TokenReference) -> Self {
+        Self { then_token, ..self }
+    }
+
+    /// Returns a new IfExpression with the given if expression
+    pub fn with_if_expression(self, if_expression: Expression) -> Self {
+        Self {
+            if_expression,
+            ..self
+        }
+    }
+
+    /// Returns a new If with the given list of `elseif` expressions
+    pub fn with_else_if(self, else_if_expressions: Option<Vec<ElseIfExpression>>) -> Self {
+        Self {
+            else_if_expressions,
+            ..self
+        }
+    }
+
+    /// Returns a new IfExpression with the given `else` token
+    pub fn with_else_token(self, else_token: TokenReference) -> Self {
+        Self { else_token, ..self }
+    }
+
+    /// Returns a new IfExpression with the given `else` expression
+    pub fn with_else(self, else_expression: Expression) -> Self {
+        Self {
+            else_expression,
+            ..self
+        }
+    }
+}
+
+/// An elseif expression in a bigger [`IfExpression`] expression
+#[derive(Clone, Debug, Display, PartialEq, Node, Visit)]
+#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
+#[display(
+    fmt = "{}{}{}{}",
+    "else_if_token",
+    "condition",
+    "then_token",
+    "expression"
+)]
+pub struct ElseIfExpression {
+    pub(crate) else_if_token: TokenReference,
+    pub(crate) condition: Expression,
+    pub(crate) then_token: TokenReference,
+    pub(crate) expression: Expression,
+}
+
+impl ElseIfExpression {
+    /// Creates a new ElseIf from the given condition
+    pub fn new(condition: Expression, expression: Expression) -> Self {
+        Self {
+            else_if_token: TokenReference::symbol(" elseif ").unwrap(),
+            condition,
+            then_token: TokenReference::symbol(" then ").unwrap(),
+            expression,
+        }
+    }
+
+    /// The `elseif` token
+    pub fn else_if_token(&self) -> &TokenReference {
+        &self.else_if_token
+    }
+
+    /// The condition of the `elseif`, `condition` in `elseif condition then`
+    pub fn condition(&self) -> &Expression {
+        &self.condition
+    }
+
+    /// The `then` token
+    pub fn then_token(&self) -> &TokenReference {
+        &self.then_token
+    }
+
+    /// The evaluated expression of the `elseif` when condition is true
+    pub fn expression(&self) -> &Expression {
+        &self.expression
+    }
+
+    /// Returns a new ElseIfExpression with the given `elseif` token
+    pub fn with_else_if_token(self, else_if_token: TokenReference) -> Self {
+        Self {
+            else_if_token,
+            ..self
+        }
+    }
+
+    /// Returns a new ElseIfExpression with the given condition
+    pub fn with_condition(self, condition: Expression) -> Self {
+        Self { condition, ..self }
+    }
+
+    /// Returns a new ElseIfExpression with the given `then` token
+    pub fn with_then_token(self, then_token: TokenReference) -> Self {
+        Self { then_token, ..self }
+    }
+
+    /// Returns a new ElseIfExpression with the given expression
+    pub fn with_block(self, expression: Expression) -> Self {
+        Self { expression, ..self }
+    }
+}
