@@ -256,8 +256,12 @@ create_visitor!(ast: {
     #[cfg(feature = "roblox")] {
         visit_compound_assignment => CompoundAssignment,
         visit_compound_op => CompoundOp,
+        visit_else_if_expression => ElseIfExpression,
         visit_exported_type_declaration => ExportedTypeDeclaration,
         visit_generic_declaration => GenericDeclaration,
+        visit_generic_declaration_parameter => GenericDeclarationParameter,
+        visit_generic_parameter_info => GenericParameterInfo,
+        visit_if_expression => IfExpression,
         visit_indexed_type_info => IndexedTypeInfo,
         visit_type_argument => TypeArgument,
         visit_type_assertion => TypeAssertion,
